@@ -4,7 +4,7 @@
 modname = 'brainomics'
 distname = 'cubicweb-brainomics'
 
-numversion = (0, 4, 6)
+numversion = (0, 8, 0)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -14,12 +14,13 @@ description = 'Cube for the Brainomics Project, see http://www.brainomics.net/'
 web = 'http://www.cubicweb.org/project/%s' % distname
 
 __depends__ =  {'cubicweb': '>= 3.17.0',
-                'cubicweb-questionnaire': None,
-                'cubicweb-neuroimaging': None,
-                'cubicweb-genomics': None,
-                'cubicweb-medicalexp': None,
+                'cubicweb-questionnaire': '>= 0.5.0',
+                'cubicweb-neuroimaging': '>= 0.5.0',
+                'cubicweb-genomics': '>= 0.7.0',
+                'cubicweb-medicalexp': '>= 0.8.0',
                 'cubicweb-card': None,
-                'cubicweb-bootstrap': None,  # should be '< 0.2.0'
+                'cubicweb-bootstrap': '>= 0.4.0',
+                'cubicweb-squareui': '>= 0.2.0',
                 'cubicweb-comment': None,
                 'cubicweb-jqplot': None,
                 }
