@@ -29,7 +29,7 @@ from cubes.brainomics.importers.helpers import (get_image_info, import_genes,
 ###############################################################################
 ### DATA GENERATION CONFIGURATION #############################################
 ###############################################################################
-NB_SUBJECTS = 10000
+NB_SUBJECTS = 200
 NB_SUBJECTS_GROUPS = 5
 AGE_DATETIMES = (datetime.strptime('1/1/1930 1:30 PM', '%m/%d/%Y %I:%M %p'),
                  datetime.strptime('1/1/1990 4:50 AM', '%m/%d/%Y %I:%M %p'))
@@ -42,13 +42,15 @@ HANDEDNESS = {0: u'right', 1: u'left', 2: u'ambidextrous', 3: u'mixed'}
 ###############################################################################
 ### UTILITY FUNCTIONS #########################################################
 ###############################################################################
-def random_date(_type=None):
+def random_date(_type=None, start=None, end=None):
     """
     This function will return a random datetime between two datetime
     objects.
     From http://stackoverflow.com/questions/553303/generate-a-random-date-between-two-other-dates
     """
-    if _type=='age':
+    if start:
+        end = PROJECT_DATETIMES[1]
+    elif _type=='age':
         start, end = AGE_DATETIMES[0], AGE_DATETIMES[1]
     else:
         start, end = PROJECT_DATETIMES[0], PROJECT_DATETIMES[1]
@@ -60,8 +62,12 @@ def random_date(_type=None):
 def import_questionnaire(store, subject_eid, center_eid, study_eid,
                          questionnaire_eid, questions, label):
     date = random_date()
+    protocol = store.create_entity('Protocol', identifier=u'Demo questionnaire protocol',
+                                   start_datetime=date)
+    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
     assessment = store.create_entity('Assessment', identifier=u'%s_%s' % (label, subject_eid),
-                                     datetime=date, protocol=u'Demo questionnaire')
+                                     datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
@@ -79,25 +85,73 @@ def import_questionnaire(store, subject_eid, center_eid, study_eid,
                                           question=question, datetime=date,
                                           questionnaire_run=q_qrun.eid)
 
-def import_genomic(store, subject_eid, center_eid, study_eid, platform=None):
+def import_genomic(store, subject_eid, center_eid, study_eid, platform=None, type='SNP'):
     date = random_date()
+    protocol = store.create_entity('Protocol', identifier=u'Demo genomics protocol',
+                                   start_datetime=date)
+    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
     assessment = store.create_entity('Assessment', identifier=u'genomics_%s' % subject_eid,
-                                     datetime=date, protocol=u'Demo genomics')
+                                     datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
     measure = store.create_entity('GenomicMeasure', identifier=u'genomic_measure_%s' % subject_eid,
-                                  type=u'SNP', format=u'plink',
+                                  type=type,
                                   platform=platform if platform else None,
                                   filepath=u'%s/genomics.tar.gz' % subject_eid,
                                   completed=True, valid=True, related_study=study_eid)
     store.relate(measure.eid, 'concerns', subject_eid, subjtype='GenomicMeasure')
     store.relate(assessment.eid, 'generates', measure.eid, subjtype='Assessment')
+    return measure.eid
+
+def create_genomic_region(store):
+    """ Add a GenomicRegion """
+    start = nr.randint(1000000)
+    stop = nr.randint(start, 1000000)
+    chromosome = store.rql('Any X ORDERBY RANDOM() LIMIT 1 WHERE X is Chromosome')
+    entity = store.create_entity('GenomicRegion', start=start, stop=stop,
+                                 width=stop-start,
+                                 cytoband_start=u'q+%s' % start,
+                                 cytoband_stop=u'q+%s' % stop,
+                                 chromosome=chromosome[0][0])
+    for gene in store.rql('Any X ORDERBY RANDOM() LIMIT 100 WHERE X is Gene, '
+                          'X chromosomes Y, Y eid %(e)s', {'e': chromosome[0][0]}):
+        store.relate(entity.eid, 'genes', gene[0], subjtype='GenomicRegion')
+    return entity.eid
+
+def import_cgh_results(store, subject_eid, center_eid, study_eid, platform):
+    """ Add CGH results """
+    measure = import_genomic(store, subject_eid, center_eid, study_eid, platform, u'CGH')
+    for region in range(nr.randint(100)):
+        region = create_genomic_region(store)
+        store.create_entity('CghResult', cgh_ratio=nr.random(), log2_ratio=nr.random(),
+                            status=(u'L', u'G')[nr.randint(1)],
+                            numprobes=nr.randint(100),
+                            related_measure=measure, genomic_region=region)
+
+def import_sequencing_results(store, subject_eid, center_eid, study_eid, platform):
+    """ Add CGH results """
+    measure = import_genomic(store, subject_eid, center_eid, study_eid, platform, u'Sequencing')
+    for mutation in range(nr.randint(100)):
+        gene = store.rql('Any X ORDERBY RANDOM() LIMIT 1 WHERE X is Gene')[0][0]
+        store.create_entity('Mutation', position_in_gene=nr.randint(1000),
+                            mutation_type=u'SNP', ploidy=(u'Het', u'Hom')[nr.randint(1)],
+                            biological_classification=u'Substitution',
+                            related_measure=measure,
+                            related_gene=gene,
+                            reference_base=(u'A', u'T', u'G', u'C')[nr.randint(3)],
+                            variant_base=(u'A', u'T', u'G', u'C')[nr.randint(3)],
+                            variant_frequency=100*nr.random())
 
 def import_anat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
     date = random_date()
+    protocol = store.create_entity('Protocol', identifier=u'Demo anat protocol',
+                                   start_datetime=date)
+    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
     assessment = store.create_entity('Assessment', identifier=u'anat_%s' % subject_eid,
-                                     datetime=date, protocol=u'Demo anat')
+                                     datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
@@ -118,8 +172,12 @@ def import_anat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
 
 def import_fmri_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
     date = random_date()
+    protocol = store.create_entity('Protocol', identifier=u'Demo fmri protocol',
+                                   start_datetime=date)
+    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
     assessment = store.create_entity('Assessment', identifier=u'fmri_%s' % subject_eid,
-                                     datetime=date, protocol=u'Demo fmri')
+                                     datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
@@ -140,8 +198,12 @@ def import_fmri_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
 
 def import_constrat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2):
     date = random_date()
+    protocol = store.create_entity('Protocol', identifier=u'Demo c map protocol',
+                                   start_datetime=date)
+    store.relate(protocol.eid, 'related_study', study_eid, subtype='Protocol')
     assessment = store.create_entity('Assessment', identifier=u'cmap_%s' % subject_eid,
-                                     datetime=date, protocol=u'Demo c map')
+                                     datetime=date)
+    store.relate(assessment.eid, 'protocols', protocol.eid, subtype='Assessment')
     store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
     store.relate(center_eid, 'holds', assessment.eid)
     store.relate(subject_eid, 'concerned_by', assessment.eid)
@@ -197,8 +259,12 @@ if __name__ == '__main__':
             genes = import_genes(options.chromosomes, options.genes)
             for gene in genes:
                 print 'gene', gene['name'], gene['chromosome']
-                gene['chromosome'] = chr_map[gene['chromosome']]
+                chromosome = gene.pop('chromosome')
+                chromosome = chr_map.get(chromosome)
+                if not chromosome:
+                    continue
                 gene = store.create_entity('Gene', **gene)
+                store.relate(gene.eid, 'chromosomes', chromosome)
             store.flush()
         if options.snps:
             snps = import_snps(options.chromosomes, options.snps)
@@ -211,6 +277,8 @@ if __name__ == '__main__':
                 if ind and ind % 100000 == 0:
                     store.flush()
             store.flush()
+        else:
+            snp_eids = []
         # Platform
         platform = {'identifier': 'Affymetrix_6.0'}
         platform = store.create_entity('GenomicPlatform', **platform).eid
@@ -334,7 +402,7 @@ if __name__ == '__main__':
     ###############################################################################
     groups = []
     for ind in range(NB_SUBJECTS_GROUPS):
-        groups.append(store.create_entity('Group', identifier=u'demo_group_%s' % ind,
+        groups.append(store.create_entity('SubjectGroup', identifier=u'demo_group_%s' % ind,
                                           name=u'Demo group of subjects %s' % ind).eid)
     subjects = []
     for ind in range(NB_SUBJECTS):
@@ -376,7 +444,25 @@ if __name__ == '__main__':
     ###############################################################################
     for subject_eid in subjects:
         print subject_eid, 'genomics'
-        import_genomic(store, subject_eid, center, study, platform)
+        measure = import_genomic(store, subject_eid, center, study, platform)
+
+    store.flush()
+    store.commit()
+
+    for subject_eid in subjects:
+        # Only few subject
+        if nr.randint(10):
+            print subject_eid, 'cgh'
+            measure = import_cgh_results(store, subject_eid, center, study, platform)
+
+    store.flush()
+    store.commit()
+
+    for subject_eid in subjects:
+        # Only few subject
+        if nr.randint(10):
+            print subject_eid, 'sequencing'
+            measure = import_sequencing_results(store, subject_eid, center, study, platform)
 
     store.flush()
     store.commit()
@@ -392,3 +478,40 @@ if __name__ == '__main__':
         import_constrat_images(store, subject_eid, center, study, mri_1, mri_2)
     store.flush()
     store.commit()
+
+
+    ###############################################################################
+    ### DISEASE/THERAPY ###########################################################
+    ###############################################################################
+    diseases = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 10 '
+                                        'WHERE X is Disease')]
+    locations = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
+                                         'WHERE X is BodyLocation')]
+    drugs = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
+                                         'WHERE X is Drug')]
+    if diseases:
+        for subject_eid in subjects:
+            disease = diseases[nr.randint(len(diseases))]
+            location = locations[nr.randint(len(locations))]
+            drug = drugs[nr.randint(len(drugs))]
+            store.relate(subject_eid, 'related_diseases', disease)
+            if nr.randint(3):
+                store.relate(subject_eid, 'related_lesions', location)
+            start_date = random_date()
+            stop_date = random_date(start=start_date)
+            therapy = store.create_entity('Therapy',
+                                          identifier=u'therapy_%s' % subject_eid,
+                                           start_date=start_date,
+                                           stop_date=stop_date if nr.randint(2)
+                                          else None)
+            store.relate(therapy.eid, 'therapy_for', disease)
+            store.relate(subject_eid, 'related_therapies', therapy.eid)
+            stop_date = random_date(start=start_date)
+            drugtake = store.create_entity('DrugTake',
+                                           start_taking_date=start_date,
+                                           stop_taking_date=stop_date if nr.randint(2)
+                                           else None,
+                                           taken_in_therapy=therapy.eid,
+                                           drug=drug)
+        store.flush()
+        store.commit()
